@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2010-2011 Robert Ancell.
- * Author: Robert Ancell <robert.ancell@canonical.com>
+ * Copyright (C) 2010-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -13,8 +12,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <fcntl.h>
 #include <gcrypt.h>
 
 #include "greeter.h"
@@ -22,12 +19,12 @@
 #include "shared-data-manager.h"
 
 enum {
-    PROP_0,
-    PROP_ACTIVE_USERNAME,
+    PROP_ACTIVE_USERNAME = 1,
 };
 
 enum {
     CONNECTED,
+    DISCONNECTED,  
     CREATE_SESSION,
     START_SESSION,
     LAST_SIGNAL
@@ -83,7 +80,7 @@ struct GreeterPrivate
     guint from_greeter_watch;
 };
 
-G_DEFINE_TYPE (Greeter, greeter, SESSION_TYPE);
+G_DEFINE_TYPE (Greeter, greeter, G_TYPE_OBJECT);
 
 /* Messages from the greeter to the server */
 typedef enum
@@ -119,9 +116,43 @@ greeter_new (void)
     return g_object_new (GREETER_TYPE, NULL);
 }
 
+void
+greeter_set_file_descriptors (Greeter *greeter, int to_greeter_fd, int from_greeter_fd)
+{
+    GError *error = NULL;
+
+    g_return_if_fail (greeter != NULL);
+    g_return_if_fail (greeter->priv->to_greeter_input < 0);
+    g_return_if_fail (greeter->priv->from_greeter_output < 0);
+
+    greeter->priv->to_greeter_input = to_greeter_fd;  
+    greeter->priv->to_greeter_channel = g_io_channel_unix_new (greeter->priv->to_greeter_input);
+    g_io_channel_set_encoding (greeter->priv->to_greeter_channel, NULL, &error);
+    if (error)
+        g_warning ("Failed to set encoding on to greeter channel to binary: %s\n", error->message);
+    g_clear_error (&error);
+    greeter->priv->from_greeter_output = from_greeter_fd;
+    greeter->priv->from_greeter_channel = g_io_channel_unix_new (greeter->priv->from_greeter_output);
+    g_io_channel_set_encoding (greeter->priv->from_greeter_channel, NULL, &error);
+    if (error)
+        g_warning ("Failed to set encoding on from greeter channel to binary: %s\n", error->message);
+    g_clear_error (&error);
+    g_io_channel_set_buffered (greeter->priv->from_greeter_channel, FALSE);
+    greeter->priv->from_greeter_watch = g_io_add_watch (greeter->priv->from_greeter_channel, G_IO_IN | G_IO_HUP, read_cb, greeter);
+}
+
+void
+greeter_stop (Greeter *greeter)
+{
+    /* Stop any events occurring after we've stopped */
+    if (greeter->priv->authentication_session)
+        g_signal_handlers_disconnect_matched (greeter->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, greeter);
+}
+
 void
 greeter_set_pam_services (Greeter *greeter, const gchar *pam_service, const gchar *autologin_pam_service)
 {
+    g_return_if_fail (greeter != NULL);
     g_free (greeter->priv->pam_service);
     greeter->priv->pam_service = g_strdup (pam_service);
     g_free (greeter->priv->autologin_pam_service);
@@ -131,18 +162,21 @@ greeter_set_pam_services (Greeter *greeter, const gchar *pam_service, const gcha
 void
 greeter_set_allow_guest (Greeter *greeter, gboolean allow_guest)
 {
+    g_return_if_fail (greeter != NULL);  
     greeter->priv->allow_guest = allow_guest;
 }
 
 void
 greeter_clear_hints (Greeter *greeter)
 {
+    g_return_if_fail (greeter != NULL);
     g_hash_table_remove_all (greeter->priv->hints);
 }
 
 void
 greeter_set_hint (Greeter *greeter, const gchar *name, const gchar *value)
 {
+    g_return_if_fail (greeter != NULL);
     g_hash_table_insert (greeter->priv->hints, g_strdup (name), g_strdup (value));
 }
 
@@ -198,7 +232,7 @@ write_message (Greeter *greeter, guint8 *message, gsize message_length)
 
         status = g_io_channel_write_chars (greeter->priv->to_greeter_channel, data, data_length, &n_written, &error);
         if (error)
-            l_warning (greeter, "Error writing to greeter: %s", error->message);
+            g_warning ("Error writing to greeter: %s", error->message);
         g_clear_error (&error);
         if (status != G_IO_STATUS_NORMAL)
             return;
@@ -208,7 +242,7 @@ write_message (Greeter *greeter, guint8 *message, gsize message_length)
 
     g_io_channel_flush (greeter->priv->to_greeter_channel, &error);
     if (error)
-        l_warning (greeter, "Failed to flush data to greeter: %s", error->message);
+        g_warning ("Failed to flush data to greeter: %s", error->message);
     g_clear_error (&error);
 }
 
@@ -268,7 +302,7 @@ handle_connect (Greeter *greeter, const gchar *version, gboolean resettable)
     GHashTableIter iter;
     gpointer key, value;
 
-    l_debug (greeter, "Greeter connected version=%s resettable=%s", version, resettable ? "true" : "false");
+    g_debug ("Greeter connected version=%s resettable=%s", version, resettable ? "true" : "false");
 
     greeter->priv->resettable = resettable;
 
@@ -305,7 +339,7 @@ pam_messages_cb (Session *session, Greeter *greeter)
     messages_length = session_get_messages_length (session);
 
     /* Respond to d-bus query with messages */
-    l_debug (greeter, "Prompt greeter with %d message(s)", messages_length);
+    g_debug ("Prompt greeter with %d message(s)", messages_length);
     size = int_length () + string_length (session_get_username (session)) + int_length ();
     for (i = 0; i < messages_length; i++)
         size += int_length () + string_length (messages[i].msg);
@@ -367,6 +401,8 @@ greeter_reset (Greeter *greeter)
     GHashTableIter iter;
     gpointer key, value;
 
+    g_return_if_fail (greeter != NULL);
+
     g_hash_table_iter_init (&iter, greeter->priv->hints);
     while (g_hash_table_iter_next (&iter, &key, &value))
         length += string_length (key) + string_length (value);
@@ -386,16 +422,16 @@ authentication_complete_cb (Session *session, Greeter *greeter)
 {
     int result;
 
-    l_debug (greeter, "Authenticate result for user %s: %s", session_get_username (session), session_get_authentication_result_string (session));
+    g_debug ("Authenticate result for user %s: %s", session_get_username (session), session_get_authentication_result_string (session));
 
     result = session_get_authentication_result (session);
     if (session_get_is_authenticated (session))
     {
         if (session_get_user (session))
-            l_debug (greeter, "User %s authorized", session_get_username (session));
+            g_debug ("User %s authorized", session_get_username (session));
         else
         {
-            l_debug (greeter, "User %s authorized, but no account of that name exists", session_get_username (session));
+            g_debug ("User %s authorized, but no account of that name exists", session_get_username (session));
             result = PAM_USER_UNKNOWN;
         }
     }
@@ -412,33 +448,32 @@ reset_session (Greeter *greeter)
     {
         g_signal_handlers_disconnect_matched (greeter->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, greeter);
         session_stop (greeter->priv->authentication_session);
-        g_object_unref (greeter->priv->authentication_session);
-        greeter->priv->authentication_session = NULL;
+        g_clear_object (&greeter->priv->authentication_session);
     }
 
     greeter->priv->guest_account_authenticated = FALSE;
 }
 
 static void
-handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
+handle_authenticate (Greeter *greeter, guint32 sequence_number, const gchar *username)
 {
     const gchar *autologin_username, *service;
     gboolean is_interactive;
 
     if (username[0] == '\0')
     {
-        l_debug (greeter, "Greeter start authentication");
+        g_debug ("Greeter start authentication");
         username = NULL;
     }
     else
-        l_debug (greeter, "Greeter start authentication for %s", username);
+        g_debug ("Greeter start authentication for %s", username);
 
     reset_session (greeter);
 
     if (greeter->priv->active_username)
         g_free (greeter->priv->active_username);
     greeter->priv->active_username = g_strdup (username);
-    g_object_notify (G_OBJECT (greeter), "active-username");
+    g_object_notify (G_OBJECT (greeter), GREETER_PROPERTY_ACTIVE_USERNAME);
 
     greeter->priv->authentication_sequence_number = sequence_number;
     g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
@@ -448,8 +483,8 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
         return;
     }
 
-    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
-    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
+    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_GOT_MESSAGES, G_CALLBACK (pam_messages_cb), greeter);
+    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (authentication_complete_cb), greeter);
 
     /* Use non-interactive service for autologin user */
     autologin_username = g_hash_table_lookup (greeter->priv->hints, "autologin-user");
@@ -473,15 +508,15 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
 }
 
 static void
-handle_login_as_guest (Greeter *greeter, guint32 sequence_number)
+handle_authenticate_as_guest (Greeter *greeter, guint32 sequence_number)
 {
-    l_debug (greeter, "Greeter start authentication for guest account");
+    g_debug ("Greeter start authentication for guest account");
 
     reset_session (greeter);
 
     if (!greeter->priv->allow_guest)
     {
-        l_debug (greeter, "Guest account is disabled");
+        g_debug ("Guest account is disabled");
         send_end_authentication (greeter, sequence_number, "", PAM_USER_UNKNOWN);
         return;
     }
@@ -526,17 +561,17 @@ get_remote_session_service (const gchar *session_name)
 }
 
 static void
-handle_login_remote (Greeter *greeter, const gchar *session_name, const gchar *username, guint32 sequence_number)
+handle_authenticate_remote (Greeter *greeter, const gchar *session_name, const gchar *username, guint32 sequence_number)
 {
     gchar *service;
 
     if (username[0] == '\0')
     {
-        l_debug (greeter, "Greeter start authentication for remote session %s", session_name);
+        g_debug ("Greeter start authentication for remote session %s", session_name);
         username = NULL;
     }
     else
-        l_debug (greeter, "Greeter start authentication for remote session %s as user %s", session_name, username);
+        g_debug ("Greeter start authentication for remote session %s as user %s", session_name, username);
 
     reset_session (greeter);
 
@@ -552,8 +587,8 @@ handle_login_remote (Greeter *greeter, const gchar *session_name, const gchar *u
     g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
     if (greeter->priv->authentication_session)
     {
-        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
-        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
+        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_GOT_MESSAGES, G_CALLBACK (pam_messages_cb), greeter);
+        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (authentication_complete_cb), greeter);
 
         /* Run the session process */
         session_set_pam_service (greeter->priv->authentication_session, service);
@@ -601,7 +636,7 @@ handle_continue_authentication (Greeter *greeter, gchar **secrets)
         return;
     }
 
-    l_debug (greeter, "Continue authentication");
+    g_debug ("Continue authentication");
 
     /* Build response */
     response = calloc (messages_length, sizeof (struct pam_response));
@@ -631,7 +666,7 @@ handle_cancel_authentication (Greeter *greeter)
     if (greeter->priv->authentication_session == NULL)
         return;
 
-    l_debug (greeter, "Cancel authentication");
+    g_debug ("Cancel authentication");
     reset_session (greeter);
 }
 
@@ -656,15 +691,15 @@ handle_start_session (Greeter *greeter, const gchar *session)
     if (greeter->priv->guest_account_authenticated || session_get_is_authenticated (greeter->priv->authentication_session))
     {
         if (session)
-            l_debug (greeter, "Greeter requests session %s", session);
+            g_debug ("Greeter requests session %s", session);
         else
-            l_debug (greeter, "Greeter requests default session");
+            g_debug ("Greeter requests default session");
         greeter->priv->start_session = TRUE;
         g_signal_emit (greeter, signals[START_SESSION], 0, session_type, session, &result);
     }
     else
     {
-        l_debug (greeter, "Ignoring start session request, user is not authorized");
+        g_debug ("Ignoring start session request, user is not authorized");
         result = FALSE;
     }
 
@@ -680,18 +715,18 @@ handle_set_language (Greeter *greeter, const gchar *language)
 
     if (!greeter->priv->guest_account_authenticated && !session_get_is_authenticated (greeter->priv->authentication_session))
     {
-        l_debug (greeter, "Ignoring set language request, user is not authorized");
+        g_debug ("Ignoring set language request, user is not authorized");
         return;
     }
 
     // FIXME: Could use this
     if (greeter->priv->guest_account_authenticated)
     {
-        l_debug (greeter, "Ignoring set language request for guest user");
+        g_debug ("Ignoring set language request for guest user");
         return;
     }
 
-    l_debug (greeter, "Greeter sets language %s", language);
+    g_debug ("Greeter sets language %s", language);
     user = session_get_user (greeter->priv->authentication_session);
     user_set_language (user, language);
 }
@@ -703,7 +738,7 @@ handle_ensure_shared_dir (Greeter *greeter, const gchar *username)
     guint8 message[MAX_MESSAGE_LENGTH];
     gsize offset = 0;
 
-    l_debug (greeter, "Greeter requests data directory for user %s", username);
+    g_debug ("Greeter requests data directory for user %s", username);
 
     dir = shared_data_manager_ensure_user_dir (shared_data_manager_get_instance (), username);
 
@@ -721,7 +756,7 @@ read_int (Greeter *greeter, gsize *offset)
     guint8 *buffer;
     if (greeter->priv->n_read - *offset < sizeof (guint32))
     {
-        l_warning (greeter, "Not enough space for int, need %zu, got %zu", sizeof (guint32), greeter->priv->n_read - *offset);
+        g_warning ("Not enough space for int, need %zu, got %zu", sizeof (guint32), greeter->priv->n_read - *offset);
         return 0;
     }
     buffer = greeter->priv->read_buffer + *offset;
@@ -741,7 +776,7 @@ get_message_length (Greeter *greeter)
 
     if (HEADER_SIZE + payload_length < HEADER_SIZE)
     {
-        l_warning (greeter, "Payload length of %u octets too long", payload_length);
+        g_warning ("Payload length of %u octets too long", payload_length);
         return HEADER_SIZE;
     }
 
@@ -757,7 +792,7 @@ read_string_full (Greeter *greeter, gsize *offset, void* (*alloc_fn)(size_t n))
     length = read_int (greeter, offset);
     if (greeter->priv->n_read - *offset < length)
     {
-        l_warning (greeter, "Not enough space for string, need %u, got %zu", length, greeter->priv->n_read - *offset);
+        g_warning ("Not enough space for string, need %u, got %zu", length, greeter->priv->n_read - *offset);
         return g_strdup ("");
     }
 
@@ -799,8 +834,9 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 
     if (condition == G_IO_HUP)
     {
-        l_debug (greeter, "Greeter closed communication channel");
+        g_debug ("Greeter closed communication channel");
         greeter->priv->from_greeter_watch = 0;
+        g_signal_emit (greeter, signals[DISCONNECTED], 0);
         return FALSE;
     }
 
@@ -821,9 +857,16 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
                                       &n_read,
                                       &error);
     if (error)
-        l_warning (greeter, "Error reading from greeter: %s", error->message);
+        g_warning ("Error reading from greeter: %s", error->message);
     g_clear_error (&error);
-    if (status != G_IO_STATUS_NORMAL)
+    if (status == G_IO_STATUS_EOF)
+    {
+        g_debug ("Greeter closed communication channel");
+        greeter->priv->from_greeter_watch = 0;
+        g_signal_emit (greeter, signals[DISCONNECTED], 0);
+        return FALSE;
+    }
+    else if (status != G_IO_STATUS_NORMAL)
         return TRUE;
 
     greeter->priv->n_read += n_read;
@@ -857,25 +900,25 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
     case GREETER_MESSAGE_AUTHENTICATE:
         sequence_number = read_int (greeter, &offset);
         username = read_string (greeter, &offset);
-        handle_login (greeter, sequence_number, username);
+        handle_authenticate (greeter, sequence_number, username);
         g_free (username);
         break;
     case GREETER_MESSAGE_AUTHENTICATE_AS_GUEST:
         sequence_number = read_int (greeter, &offset);
-        handle_login_as_guest (greeter, sequence_number);
+        handle_authenticate_as_guest (greeter, sequence_number);
         break;
     case GREETER_MESSAGE_AUTHENTICATE_REMOTE:
         sequence_number = read_int (greeter, &offset);
         session_name = read_string (greeter, &offset);
         username = read_string (greeter, &offset);
-        handle_login_remote (greeter, session_name, username, sequence_number);
+        handle_authenticate_remote (greeter, session_name, username, sequence_number);
         break;
     case GREETER_MESSAGE_CONTINUE_AUTHENTICATION:
         n_secrets = read_int (greeter, &offset);
         max_secrets = (G_MAXUINT32 - 1) / sizeof (gchar *);
         if (n_secrets > max_secrets)
         {
-            l_warning (greeter, "Array length of %u elements too long", n_secrets);
+            g_warning ("Array length of %u elements too long", n_secrets);
             greeter->priv->from_greeter_watch = 0;
             return FALSE;
         }
@@ -907,7 +950,7 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
         g_free (username);
         break;
     default:
-        l_warning (greeter, "Unknown message from greeter: %d", id);
+        g_warning ("Unknown message from greeter: %d", id);
         break;
     }
 
@@ -924,10 +967,18 @@ greeter_get_guest_authenticated (Greeter *greeter)
 }
 
 Session *
-greeter_get_authentication_session (Greeter *greeter)
+greeter_take_authentication_session (Greeter *greeter)
 {
+    Session *session;
+
     g_return_val_if_fail (greeter != NULL, NULL);
-    return greeter->priv->authentication_session;
+
+    session = greeter->priv->authentication_session;
+    if (greeter->priv->authentication_session)
+        g_signal_handlers_disconnect_matched (greeter->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, greeter);
+    greeter->priv->authentication_session = NULL;
+
+    return session;
 }
 
 gboolean
@@ -951,60 +1002,6 @@ greeter_get_active_username (Greeter *greeter)
     return greeter->priv->active_username;
 }
 
-static gboolean
-greeter_start (Session *session)
-{
-    Greeter *greeter = GREETER (session);
-    int to_greeter_pipe[2], from_greeter_pipe[2];
-    int to_greeter_output, from_greeter_input;
-    gboolean result = FALSE;
-    gchar *value;
-    GError *error = NULL;
-
-    /* Create a pipe to talk with the greeter */
-    if (pipe (to_greeter_pipe) != 0 || pipe (from_greeter_pipe) != 0)
-    {
-        g_warning ("Failed to create pipes: %s", strerror (errno));
-        return FALSE;
-    }
-    to_greeter_output = to_greeter_pipe[0];
-    greeter->priv->to_greeter_input = to_greeter_pipe[1];
-    greeter->priv->to_greeter_channel = g_io_channel_unix_new (greeter->priv->to_greeter_input);
-    g_io_channel_set_encoding (greeter->priv->to_greeter_channel, NULL, &error);
-    if (error)
-        g_warning ("Failed to set encoding on to greeter channel to binary: %s\n", error->message);
-    g_clear_error (&error);
-    greeter->priv->from_greeter_output = from_greeter_pipe[0];
-    from_greeter_input = from_greeter_pipe[1];
-    greeter->priv->from_greeter_channel = g_io_channel_unix_new (greeter->priv->from_greeter_output);
-    g_io_channel_set_encoding (greeter->priv->from_greeter_channel, NULL, &error);
-    if (error)
-        g_warning ("Failed to set encoding on from greeter channel to binary: %s\n", error->message);
-    g_clear_error (&error);
-    g_io_channel_set_buffered (greeter->priv->from_greeter_channel, FALSE);
-    greeter->priv->from_greeter_watch = g_io_add_watch (greeter->priv->from_greeter_channel, G_IO_IN | G_IO_HUP, read_cb, greeter);
-
-    /* Let the greeter session know how to communicate with the daemon */
-    value = g_strdup_printf ("%d", from_greeter_input);
-    session_set_env (SESSION (greeter), "LIGHTDM_TO_SERVER_FD", value);
-    g_free (value);
-    value = g_strdup_printf ("%d", to_greeter_output);
-    session_set_env (SESSION (greeter), "LIGHTDM_FROM_SERVER_FD", value);
-    g_free (value);
-
-    /* Don't allow the daemon end of the pipes to be accessed in child processes */
-    fcntl (greeter->priv->to_greeter_input, F_SETFD, FD_CLOEXEC);
-    fcntl (greeter->priv->from_greeter_output, F_SETFD, FD_CLOEXEC);
-
-    result = SESSION_CLASS (greeter_parent_class)->start (session);
-
-    /* Close the session ends of the pipe */
-    close (to_greeter_output);
-    close (from_greeter_input);
-
-    return result;
-}
-
 static Session *
 greeter_real_create_session (Greeter *greeter)
 {
@@ -1017,18 +1014,6 @@ greeter_real_start_session (Greeter *greeter, SessionType type, const gchar *ses
     return FALSE;
 }
 
-static void
-greeter_stop (Session *session)
-{
-    Greeter *greeter = GREETER (session);
-
-    /* Stop any events occurring after we've stopped */
-    if (greeter->priv->authentication_session)
-        g_signal_handlers_disconnect_matched (greeter->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, greeter);
-
-    SESSION_CLASS (greeter_parent_class)->stop (session);
-}
-
 static void
 greeter_init (Greeter *greeter)
 {
@@ -1036,14 +1021,14 @@ greeter_init (Greeter *greeter)
     greeter->priv->read_buffer = secure_malloc (greeter, HEADER_SIZE);
     greeter->priv->hints = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
     greeter->priv->use_secure_memory = config_get_boolean (config_get_instance (), "LightDM", "lock-memory");
+    greeter->priv->to_greeter_input = -1;
+    greeter->priv->from_greeter_output = -1;
 }
 
 static void
 greeter_finalize (GObject *object)
 {
-    Greeter *self;
-
-    self = GREETER (object);
+    Greeter *self = GREETER (object);
 
     g_free (self->priv->pam_service);
     g_free (self->priv->autologin_pam_service);
@@ -1098,19 +1083,16 @@ greeter_get_property (GObject    *object,
 static void
 greeter_class_init (GreeterClass *klass)
 {
-    SessionClass *session_class = SESSION_CLASS (klass);
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
     klass->create_session = greeter_real_create_session;
     klass->start_session = greeter_real_start_session;
-    session_class->start = greeter_start;
-    session_class->stop = greeter_stop;
     object_class->finalize = greeter_finalize;
     object_class->get_property = greeter_get_property;
     object_class->set_property = greeter_set_property;
 
     signals[CONNECTED] =
-        g_signal_new ("connected",
+        g_signal_new (GREETER_SIGNAL_CONNECTED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, connected),
@@ -1118,8 +1100,17 @@ greeter_class_init (GreeterClass *klass)
                       NULL,
                       G_TYPE_NONE, 0);
 
+    signals[DISCONNECTED] =
+        g_signal_new (GREETER_SIGNAL_DISCONNECTED,
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (GreeterClass, disconnected),
+                      NULL, NULL,
+                      NULL,
+                      G_TYPE_NONE, 0);
+
     signals[CREATE_SESSION] =
-        g_signal_new ("create-session",
+        g_signal_new (GREETER_SIGNAL_CREATE_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, create_session),
@@ -1129,7 +1120,7 @@ greeter_class_init (GreeterClass *klass)
                       SESSION_TYPE, 0);
 
     signals[START_SESSION] =
-        g_signal_new ("start-session",
+        g_signal_new (GREETER_SIGNAL_START_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, start_session),
@@ -1140,8 +1131,8 @@ greeter_class_init (GreeterClass *klass)
 
     g_object_class_install_property (object_class,
                                      PROP_ACTIVE_USERNAME,
-                                     g_param_spec_string ("active-username",
-                                                          "active-username",
+                                     g_param_spec_string (GREETER_PROPERTY_ACTIVE_USERNAME,
+                                                          GREETER_PROPERTY_ACTIVE_USERNAME,
                                                           "Active username",
                                                           NULL,
                                                           G_PARAM_READABLE));
